@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * Hitori
- * Copyright (C) Philip Withnall 2007-2008 <philip@tecnocode.co.uk>
+ * Copyright (C) Philip Withnall 2007-2009 <philip@tecnocode.co.uk>
  * 
  * Hitori is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,6 +33,19 @@
 #define HINT_FLASHES 6
 #define HINT_INTERVAL 500
 
+/* Declarations for GtkBuilder */
+gboolean hitori_expose_cb (GtkWidget *drawing_area, GdkEventExpose *event, Hitori *hitori);
+gboolean hitori_button_release_cb (GtkWidget *drawing_area, GdkEventButton *event, Hitori *hitori);
+void hitori_destroy_cb (GtkWindow *window, Hitori *hitori);
+void hitori_new_game_cb (GtkAction *action, Hitori *hitori);
+void hitori_hint_cb (GtkAction *action, Hitori *hitori);
+void hitori_undo_cb (GtkAction *action, Hitori *hitori);
+void hitori_redo_cb (GtkAction *action, Hitori *hitori);
+void hitori_quit_cb (GtkAction *action, Hitori *hitori);
+void hitori_contents_cb (GtkAction *action, Hitori *hitori);
+void hitori_about_cb (GtkAction *action, Hitori *hitori);
+void hitori_board_size_cb (GtkRadioAction *action, GtkRadioAction *current, Hitori *hitori);
+
 GtkWidget *
 hitori_create_interface (Hitori *hitori)
 {
@@ -48,8 +61,8 @@ hitori_create_interface (Hitori *hitori)
 				GTK_DIALOG_MODAL,
 				GTK_MESSAGE_ERROR,
 				GTK_BUTTONS_OK,
-				_("UI file \"%s/hitori/hitori.ui\" could not be loaded."), PACKAGE_DATA_DIR);
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), error->message);
+				_("UI file “%s” could not be loaded"), PACKAGE_DATA_DIR"/hitori/hitori.ui");
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", error->message);
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
 
@@ -98,7 +111,7 @@ hitori_draw_board (Hitori *hitori, cairo_t *cr, gboolean check_win)
 		has_won = TRUE;
 	}
 
-	gdk_drawable_get_size (GDK_DRAWABLE (hitori->drawing_area->window), &area_width, &area_height);
+	gdk_drawable_get_size (GDK_DRAWABLE (gtk_widget_get_window (hitori->drawing_area)), &area_width, &area_height);
 	style = gtk_widget_get_style (hitori->drawing_area);
 
 	/* Clamp the width/height to the minimum */
@@ -199,13 +212,12 @@ hitori_draw_board (Hitori *hitori, cairo_t *cr, gboolean check_win)
 	}
 
 	if (has_won == TRUE) {
+		GtkWidget *dialog;
+
 		/* Tell the user they've won, and don't draw any hints */
 		hitori_disable_events (hitori);
-		GtkWidget *dialog = gtk_message_dialog_new (NULL,
-				GTK_DIALOG_MODAL,
-				GTK_MESSAGE_INFO,
-				GTK_BUTTONS_OK,
-				_("You've won!"));
+		dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
+						 _("You've won!"));
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
 		return;
@@ -237,9 +249,9 @@ hitori_draw_board_simple (Hitori *hitori, gboolean check_win, gboolean clear_fir
 	cairo_t *cr;
 
 	if (clear_first)
-		gdk_window_clear (hitori->drawing_area->window);
+		gdk_window_clear (gtk_widget_get_window (hitori->drawing_area));
 
-	cr = gdk_cairo_create (GDK_DRAWABLE (hitori->drawing_area->window));
+	cr = gdk_cairo_create (GDK_DRAWABLE (gtk_widget_get_window (hitori->drawing_area)));
 	hitori_draw_board (hitori, cr, check_win);
 	cairo_destroy (cr);
 }
@@ -249,7 +261,7 @@ hitori_expose_cb (GtkWidget *drawing_area, GdkEventExpose *event, Hitori *hitori
 {
 	cairo_t *cr;
 
-	cr = gdk_cairo_create (GDK_DRAWABLE (hitori->drawing_area->window));
+	cr = gdk_cairo_create (GDK_DRAWABLE (gtk_widget_get_window (hitori->drawing_area)));
 	cairo_rectangle (cr, event->area.x, event->area.y,
 				event->area.width, event->area.height);
 	cairo_clip (cr);
@@ -271,7 +283,7 @@ hitori_button_release_cb (GtkWidget *drawing_area, GdkEventButton *event, Hitori
 	if (hitori->processing_events == FALSE)
 		return FALSE;
 
-	gdk_drawable_get_size (GDK_DRAWABLE (hitori->drawing_area->window), &width, &height);
+	gdk_drawable_get_size (GDK_DRAWABLE (gtk_widget_get_window (hitori->drawing_area)), &width, &height);
 
 	/* Clamp the width/height to the minimum */
 	if (height < width)
@@ -319,6 +331,9 @@ hitori_button_release_cb (GtkWidget *drawing_area, GdkEventButton *event, Hitori
 	/* Stop any current hints */
 	hitori->hint_status = HINT_FLASHES;
 
+	if (hitori->debug)
+		g_debug ("Stopping all current hints.");
+
 	/* Redraw */
 	hitori_draw_board_simple (hitori, recheck, FALSE);
 
@@ -338,19 +353,24 @@ hitori_new_game_cb (GtkAction *action, Hitori *hitori)
 }
 
 static gboolean
-hitori_update_hint (gpointer user_data)
+hitori_update_hint (Hitori *hitori)
 {
-	Hitori *hitori;
 	cairo_t *cr;
 	gint area_width, area_height;
 	guint board_width, board_height;
 	gfloat cell_size;
 
-	hitori = (Hitori*) user_data;
+	/* Check to see if hinting's been stopped by a cell being changed (race condition) */
+	if (hitori->hint_status >= HINT_FLASHES)
+		return FALSE;
+
 	hitori->hint_status++;
 
+	if (hitori->debug)
+		g_debug ("Updating hint status to %u.", hitori->hint_status);
+
 	/* Calculate the area to redraw (just the hinted cell, hopefully) */
-	gdk_drawable_get_size (GDK_DRAWABLE (hitori->drawing_area->window), &area_width, &area_height);
+	gdk_drawable_get_size (GDK_DRAWABLE (gtk_widget_get_window (hitori->drawing_area)), &area_width, &area_height);
 
 	/* Clamp the width/height to the minimum */
 	if (area_height < area_width) {
@@ -364,7 +384,7 @@ hitori_update_hint (gpointer user_data)
 	cell_size = board_width / hitori->board_size;
 
 	/* Centre the board */
-	cr = gdk_cairo_create (GDK_DRAWABLE (hitori->drawing_area->window));
+	cr = gdk_cairo_create (GDK_DRAWABLE (gtk_widget_get_window (hitori->drawing_area)));
 	cairo_save (cr);
 
 	hitori->drawing_area_x_offset = (area_width - board_width) / 2;
@@ -394,11 +414,15 @@ hitori_hint_cb (GtkAction *action, Hitori *hitori)
 
 			if (status <= MAX (CELL_SHOULD_BE_PAINTED, CELL_PAINTED) &&
 			    status > 0) {
+				if (hitori->debug)
+					g_debug ("Beginning hinting in cell (%u,%u).", iter.x, iter.y);
+
 				/* Set up the cell for hinting */
 				hitori->hint_status = 0;
 				hitori->hint_position = iter;
-				g_timeout_add (HINT_INTERVAL, hitori_update_hint,(gpointer) hitori);
+				g_timeout_add (HINT_INTERVAL, (GSourceFunc) hitori_update_hint, hitori);
 				hitori_update_hint ((gpointer) hitori);
+
 				return;
 			}
 		}
@@ -422,7 +446,9 @@ hitori_undo_cb (GtkAction *action, Hitori *hitori)
 			hitori->board[hitori->undo_stack->cell.x][hitori->undo_stack->cell.y].status ^= CELL_TAG2;
 			break;
 		case UNDO_NEW_GAME:
+		default:
 			/* This is just here to stop the compiler warning */
+			g_assert_not_reached ();
 			break;
 	}
 
@@ -455,7 +481,9 @@ hitori_redo_cb (GtkAction *action, Hitori *hitori)
 			hitori->board[hitori->undo_stack->cell.x][hitori->undo_stack->cell.y].status ^= CELL_TAG2;
 			break;
 		case UNDO_NEW_GAME:
+		default:
 			/* This is just here to stop the compiler warning */
+			g_assert_not_reached ();
 			break;
 	}
 
@@ -476,20 +504,15 @@ hitori_quit_cb (GtkAction *action, Hitori *hitori)
 void
 hitori_contents_cb (GtkAction *action, Hitori *hitori)
 {
-	GTimeVal current_time;
 	GError *error = NULL;
 
-	g_get_current_time (&current_time);
-
-	if (gtk_show_uri (gtk_widget_get_screen (hitori->window), "ghelp:hitori",
-			  current_time.tv_sec * 1000 + current_time.tv_usec,
-			  &error) == FALSE) {
+	if (gtk_show_uri (gtk_widget_get_screen (hitori->window), "ghelp:hitori", gtk_get_current_event_time (), &error) == FALSE) {
 		GtkWidget *dialog = gtk_message_dialog_new (NULL,
-				GTK_DIALOG_MODAL,
-				GTK_MESSAGE_ERROR,
-				GTK_BUTTONS_OK,
-				_("The help contents could not be displayed."));
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), error->message);
+							    GTK_DIALOG_MODAL,
+							    GTK_MESSAGE_ERROR,
+							    GTK_BUTTONS_OK,
+							    _("The help contents could not be displayed"));
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", error->message);
 
 		gtk_dialog_run (GTK_DIALOG (dialog));
 
@@ -529,7 +552,7 @@ hitori_about_cb (GtkAction *action, Hitori *hitori)
 
 	gtk_show_about_dialog (GTK_WINDOW (hitori->window),
 				"version", VERSION,
-				"copyright", _("Copyright \xc2\xa9 2007-2008 Philip Withnall"),
+				"copyright", _("Copyright \xc2\xa9 2007\342\200\2232010 Philip Withnall"),
 				"comments", _("A logic puzzle designed by Nikoli."),
 				"authors", authors,
 				"translator-credits", _("translator-credits"),
@@ -537,7 +560,7 @@ hitori_about_cb (GtkAction *action, Hitori *hitori)
 				"license", license,
 				"wrap-license", TRUE,
 				"website-label", _("Hitori Website"),
-				"website", "http://tecnocode.co.uk/projects/hitori",
+				"website", "http://live.gnome.org/Hitori",
 				NULL);
 
 	g_free (license);
