@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * Hitori
- * Copyright (C) Philip Withnall 2007-2008 <philip@tecnocode.co.uk>
+ * Copyright (C) Philip Withnall 2007-2009 <philip@tecnocode.co.uk>
  * 
  * Hitori is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -180,6 +180,7 @@ main (int argc, char *argv[])
 
 	const GOptionEntry options[] = {
 	        { "debug", 0, 0, G_OPTION_ARG_NONE, &debug, N_("Enable debug mode"), NULL },
+	        /* Translators: This means to choose a number as the "seed" for random number generation used when creating a board */
 	        { "seed", 0, 0, G_OPTION_ARG_INT, &seed, N_("Seed the board generation"), NULL },
 	        { NULL }
 	};
@@ -206,8 +207,8 @@ main (int argc, char *argv[])
 				GTK_DIALOG_MODAL,
 				GTK_MESSAGE_ERROR,
 				GTK_BUTTONS_OK,
-				_("Command-line options could not be parsed."));
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), error->message);
+				_("Command line options could not be parsed"));
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", error->message);
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
 
